<?php

	/**
	 * Manages the handling of category trees
	 * Provides methods to read/write a tree to/from cache or initiates to create a new tree from database 
	 */
	class CatTreeManager	{
		
		var $bCachedRequest = FALSE;
		var $oTree = NULL;
		
		function CatTreeManager($bForceUpdate=FALSE)	{
			$this->ReadTree($bForceUpdate);
		}
		
		function ReadTree($bForceUpdate=FALSE)	{
			if($bForceUpdate)	{
				$this->UpdateCache();
			} else {
				if(!$this->LoadFromCache())		{
					$this->UpdateCache();
				} else {
					$this->bCachedRequest = TRUE;
				}
			}
			return $this->oTree;			
		}
		
		function UpdateCache()	{

			$this->oTree = new CategoryTree();
			$this->oTree->BuildTree();
			
			$this->SaveToCache();
			return TRUE;
		}
				
		function SaveToCache()		{
			if(is_null($this->oTree))	{
				return FALSE;
			}
			
			$strCacheFile = $GLOBALS['PD_CACHEDIR']."ocatstruct.cch";
			$hFile = @fopen($strCacheFile,"w");
			
			if(!$hFile)	{
				return FALSE;
			} else {
				@fputs($hFile,serialize($this->oTree));
				fclose($hFile);
			}
			return TRUE;
		}
		
		function LoadFromCache()	{

			$strRaw = "";
			
			$strCacheFile = $GLOBALS['PD_CACHEDIR']."ocatstruct.cch";
			
			if(!is_file($strCacheFile) || !is_readable($strCacheFile))	{
				return FALSE;
			}
			
			$hFile = @fopen($strCacheFile,"r");
			if(!$hFile)	{
				return FALSE;
			} else {
				while($buffer=@fgets($hFile,1024))	{
					$strRaw .= $buffer;	
				}
				fclose($hFile);
				
				// Unserialize
				$this->oTree = unserialize($strRaw);
				if(!is_object($this->oTree))	{
					return FALSE;
				} else {
					return TRUE;
				}
			}
		}
			
		function GetTreeRoot()	{
			return $this->oTree->GetRoot(); 
		}

	}

	
	class TreeWorks		{

		var $oTreeRoot = NULL;
		
		// Traversed nodes
		var $arrNodes = array();
		
		function TreeWorks(&$oTreeRoot)		{
			$this->oTreeRoot = $oTreeRoot;
			$this->TraverseTree($oTreeRoot);
		}
		
		function GetNodeById($iId)		{
			foreach($this->arrNodes as $oNode)	{				
				if($oNode->iId==$iId)	{
					return $oNode;
				}
			}
			return NULL;
		}

		
		function GetIds()	{
			$arrIds = array();
			foreach($this->arrNodes as $oNode)	{			
				$arrIds[] = $oNode->iId;			
			}
			return $arrIds;
		}

		
		function TraverseTree(&$oNode)		{			
			if(!is_null($oNode))	{
				
				$this->arrNodes[] = $oNode;
				if(sizeof($oNode->arrChilds)>0)		{
					foreach($oNode->arrChilds as $oChildNode)	{
						$this->TraverseTree($oChildNode);
					}
				}
			}
			return TRUE;			
		}
		
	}
	
	
	class CategoryTreeWorks extends TreeWorks {
		
		function UpdateCategoryStatistics()	{
			
			// Get database access
			$DB = new SQLDatabase();
			if(!$DB->dbconnect())	{
				return FALSE;
			}
			
			// Insert
			$strGuid = md5(microtime());
			$arrValues = array();
			
			$sql = "INSERT INTO cf_catstats ".
				"(statcatid,statlevel,statadcount,statadcountcum,statlastin,".
				"statlastincum,statupdate,statchildcatcount,statguid) VALUES ";
			
			foreach($this->arrNodes as $oNode)	{
				
				if(!is_null($oNode->GetId()))	{
					$strValue = "(".$oNode->iId.",".$oNode->iDepthLevel.",".
						$oNode->iCount.",".$oNode->iCountCum.",'".
						TimeToMysql($oNode->dtLastIn)."','".TimeToMysql($oNode->dtLastInCum)."',".
						"now(),".$oNode->iFollowingCats.",'".$strGuid."')";			
					$arrValues[] = $strValue;
				}
			}
			$sql .= implode(",",$arrValues);
			$iRet = $DB->dbinsert($sql);

			if($iRet>0)	{
				// Delete old statistics
				$sql = "DELETE FROM cf_catstats ".
					"WHERE statguid != '".$strGuid."'";
				$DB->dbdelete($sql);
			}
			return TRUE;
		}
		
		
		function GetCategoryForSelect($iSelected=-1,$bDummy=FALSE)	{
			$strRet = "";

			if($bDummy)		{
				$strRet.= "<option value=\"-1\">----</option>";
			}
			
			foreach($this->arrNodes as $oNode)	{
				
				if(!is_null($oNode->iId))	{
					$strRet.= "<option value=\"".$oNode->iId."\" ".
						($iSelected==$oNode->iId?"selected ":"").
						"class=\"level".$oNode->iDepthLevel."\">".
						str_pad("",($oNode->iDepthLevel-1)*3,"-- ",STR_PAD_LEFT).
						$oNode->strName.
						"</option>\r\n";				
				}
			}
			return $strRet;		
		}
		
	}
		
	
	class Node	{
	
		var $oPrev = NULL;
		var $arrChilds = array();
		
		var $iId = NULL;
		var $iDepthLevel = 0;

		var $strName = "";
		var $strDesc = "";
		var $strLogo = "";
		var $bAgeRestricted = FALSE;
			
		var $dtLastIn = 0;
		var $dtLastInCum = 0;
		
		var $iCount = 0;
		var $iCountCum = 0;
		
		var $iFollowingCats = 0;
	
		
		function Node()	{
			$this->dtLastIn = mktime(0,0,0,1,1,2000);
			$this->dtLastInCum = $this->dtLastIn;
		}
		
		function GetId()	{
			return $this->iId;
		}
		
		function SetPredecessor(&$oNode)	{
			$this->oPrev =& $oNode;
			return TRUE;
		}
		
		function &GetPredecessor()	{
			return $this->oPrev; 
		}
				
		function AddChild(&$oNode)	{
			$this->arrChilds[] =& $oNode;
			return TRUE;
		}
		
		function AddCountFollowing($iCount)	{
			$this->iCountCum+=$iCount;
			return TRUE;
		}
		
		function AddCatFollowing($iCount)	{
			$this->iFollowingCats += $iCount;
			return TRUE;
		}
		
		function SetLastInFollowing($dtLastIn)	{
			if($this->dtLastInCum < $dtLastIn)	{
				$this->dtLastInCum = $dtLastIn;
			}
			return TRUE;
		}
		
		function GetDepthLevel()	{	return $this->iDepthLevel; }
		
	}


	class CategoryTree	{

		// protected
		var $oRoot = NULL;

		// private
		var $arrCats = array();
		
			
		function CategoryTree()	{}
		
		function &GetRoot()	{
			return $this->oRoot;
		}
		
		function BuildTree()	{
			$this->GetArticleCounts();
			
			$this->oRoot =& new Node();
			$this->AddSubNodes($this->oRoot);

		}
		
		function DebugOut()		{
			echo "<pre>";
			print_r($this->oRoot);
			echo "</pre>";			
		}
		
		function GetArticleCounts()		{

			// Get database access
			$DB = new SQLDatabase();
			if(!$DB->dbconnect())	{
				return FALSE;
			}
			
			$sql = "SELECT adcat, COUNT(*) AS Counter, ".
				"MAX(adindate) as LastIn ".
				"FROM cf_ads ".
				"WHERE 1=1 ".
				"GROUP BY adcat ".
				"ORDER BY adcat";
			$dataCount = $DB->dbselect($sql);

			if(!is_array($dataCount) || sizeof($dataCount)==0)	{
				$this->arrCats = array();				
			} else {
				foreach($dataCount as $element)	{
					$this->arrCats[$element['adcat']] = 
						array(	"COUNTER"	=> $element['Counter'],
								"LASTIN"	=> $element['LastIn']		);
				}
			}
			return TRUE;			
		}
		
		function AddSubNodes(&$oParentNode)	{

			// Get database access
			$DB = new SQLDatabase();
			if(!$DB->dbconnect())	{
				return FALSE;
			}
			
			// Build sql
			$sql = "SELECT catid, catname, catdesc, caticonlink, catagerestricted, catpresort ".
				"FROM cf_categories ".
				"WHERE ".(is_null($oParentNode->GetId())?"ISNULL(catprev)":"catprev=".$oParentNode->GetId())." ".
				"ORDER BY catpresort DESC, catid";
			$data = $DB->dbselect($sql);

			if(!is_array($data))	{
				return FALSE;
			} elseif(sizeof($data)==0)	{
				// Node has no childs to add
				return TRUE;
			} else {

				// Add child categories
				foreach($data as $element)	{
					
					$oNode =& new Node();
					$oNode->SetPredecessor($oParentNode);
					
					$oNode->iId = $element['catid'];
					$oNode->strName = $element['catname'];
					$oNode->strDesc = $element['catdesc'];
					$oNode->strLogo = $element['caticonlink'];
					$oNode->bAgeRestricted = ($element['catagerestricted']==1?TRUE:FALSE);
					
					// Set recursion depth
					$oNode->iDepthLevel = $oParentNode->GetDepthLevel();
					$oNode->iDepthLevel++;
										
					// Get the offer count for the selected element
					if(!array_key_exists($element['catid'],$this->arrCats))	{
						$oNode->dtLastIn = mktime(0,0,0,1,1,2000);
						$oNode->iCount = 0;
						$oNode->iFollowingCount = 0;
					} else {
							
						$oNode->dtLastIn = MysqlToTime($this->arrCats[$element['catid']]['LASTIN']);
						$oNode->iCount = $this->arrCats[$element['catid']]['COUNTER'];	
						
						// Promote found data to previous elements
						$this->AddCounter($oParentNode,$oNode->iCount,$oNode->dtLastIn);
					}

					$oParentNode->AddChild($oNode);
					$oParentNode->AddCatFollowing(1);
					
					// Add all childs of found node recursively
					$this->AddSubNodes($oNode);
				
				}
			}
			return TRUE;
		}
		
		function AddCounter(&$oParentNode, $iCount, $dtLastIn)	{
			
			if(!is_null($oParentNode))	{
				
				$oParentNode->AddCountFollowing($iCount);
				$oParentNode->SetLastInFollowing($dtLastIn);

				$oGrandParent =& $oParentNode->GetPredecessor();
				if(!is_null($oGrandParent))	{
					$this->AddCounter($oGrandParent,$iCount,$dtLastIn);
				}
			}
			return TRUE;
		}
		
	}

?>